package keskustelubotti;

import java.util.*;
import java.io.*;

/**
 * Botin sanavarasto, kaikki botin osaamat sanat.
 *
 * Sanavarasto tallennetaan SEK Vectoriin (vakioaikainen indeksihaku), ETT HashSetiin
 * (vakioaikainen contains-kysely), koska tm lienee tehokkain ja tilaasstvin thn menness
 * keksimni ratkaisu, verrattuna esim. sanojen mappaaminen Integer-olioihin ja HashMapiin tallentaminen.
 *
 * Pelkk Vector ei riit, koska joka sanan lisyksell tytyy tarkistaa, ettei sana ole ennestn
 * varastossa, ja Vectorilla tm vaatisi lineaarisen ajan.
 *
 * @author Samuli Kaipiainen
 * @version 0.1
 */

public class Sanavarasto
{
	Vector sanatv;
	HashSet sanath;

	/**
	 * Luo uuden, tyhjn sanavaraston
	 */
	public Sanavarasto()
	{
		this.sanatv = new Vector();
		this.sanath = new HashSet();
	}

	/**
	 * Luo uuden sanavaraston, lukee sanat tiedostosta lueTiedostosta-metodilla
	 *
	 * @param sanatiedosto Tiedosto, josta luetaan
	 */
	public Sanavarasto(String sanatiedosto)
	{
		this();
		this.lueTiedostosta(sanatiedosto);
	}

	/**
	 * Lis annetun sanan sanavarastoon (jollei sit ole ennestn)
	 *
	 *
	 * @param sana Listtv sana
	 * @return true, jos sana listtiin, false jos sana oli jo varastossa
	 */
	public boolean lisSana(String sana)
	{
		if (sanath.contains(sana)) return false;

		sanatv.add(sana);
		sanath.add(sana);
		return true;
	}

	/**
	 * @return Kuinka monta sanaa sanavarastossa on
	 */
	public int sanoja()
	{
		return sanatv.size();
	}

	/**
	 * @return Onko sanavarasto tyhj
	 */
	 public boolean onTyhj()
	 {
		 return sanatv.isEmpty();
	 }

	/**
	 * @return Satunnainen sanavaraston sana; null jos sanavarasto on tyhj
	 */
	public String annaSatunnainenSana()
	{
		if (!onTyhj()) return (String) sanatv.get((int) (Main.random() * sanoja()));
		else return null;
	}

	/**
	 * Kirjoittaa sanavaraston tiedostoon, yksi sana riville
	 *
	 * @param tiedosto Tiedosto johon kirjoitetaan
	 * @return true jos kaikki onnistui, false jos ongelmaa jossain vaiheessa
	 */
	public boolean tallennaTiedostoon(String tiedosto)
	{
		Tiedostokirjoitin t = new Tiedostokirjoitin(tiedosto);

		if (!t.avattuOK()) return false;

		for (int n = 0; n < sanoja(); n++)
			t.seuraavaRivi((String) sanatv.get(n));

		return t.sulje();
	}

	/**
	 * Lukee sanavaraston tiedostosta, yksi sana rivilt, lis olemassaolevaan sanavarastoon,
	 * ei lis sanavarastossa valmiiksi olevia sanoja
	 *
	 * @param tiedosto Tiedosto josta luetaan
	 * @return true jos kaikki onnistui, false jos ongelmia jossain vaiheessa
	 */
	public boolean lueTiedostosta(String tiedosto)
	{
		Tiedostolukija t = new Tiedostolukija(tiedosto);

		if (!t.avattuOK()) return false;

		for (String s = t.seuraavaRivi(); s != null; s = t.seuraavaRivi())
			lisSana(s);

		return t.sulje();
	}
}
